module Graphos
  module Weighted

    ##
    # This class represents a node in a weighted graph

    class Node
      attr_reader :index
      def initialize index, size=0
        @index = index
        @edges = Array.new(size||0)
        @edge_count = 0
      end

      def edges
        return [] if @edge_count == 0
        @edges[0..@edge_count-1]
      end

      def add_edge to, weight
        # Does a O(n) check deleting existing edges
        current_idx = edges.index{|n| n.to == to}
        if current_idx
          @edges[current_idx] = Edge.new(self,to,weight)
        else
          edge = Edge.new(self, to, weight)
          if(@edges.size > @edge_count)
            @edges[@edge_count] = edge
          else
            @edges << edge
          end
          @edge_count += 1
        end
      end

      def degree
        edges.inject(0){|sum,e| sum+e.weight }
      end

      def neighbor_of? index
        edges.any? {|node| node.to.index == index }
      end

      def neighbors
        edges.map{|edge| edge.to}
      end

      def edge to
        edges.lazy.select{|e| e.to.index == to}.first
      end
    end

  end
end
